@@ -6,6 +6,7 @@ use warnings;
 use base 'Nitesi::Object::Singleton';
 
 use Nitesi::Class;
+use Nitesi::Account::Password;
 use ACL::Lite; 
 
 =head1 NAME
@@ -26,6 +27,10 @@ Nitesi::Account::Manager - Account Manager for Nitesi Shop Machine
 
     $acct->logout();
 
+    if ($acct->exists('shopper@nitesi.biz')) {
+        $acct->password('nevairbe', 'shopper@nitesi.biz');
+    }
+    
 =cut
 
 my @providers;
@@ -42,6 +47,8 @@ sub init {
     my ($class, $instance, %args) = @_;
     my ($ret, @list, $init);
 
+    $instance->{password} = Nitesi::Account::Password->instance;
+
     if ($args{provider_sub}) {
 	# retrieve list of providers
 	$ret = $args{provider_sub}->();
@@ -56,9 +63,8 @@ sub init {
 
 	# instantiate provider objects
 	for $init (@list) {
-	    push @providers, Nitesi::Class->instantiate(@$init);
+	    push @providers, Nitesi::Class->instantiate(@$init, crypt => $instance->{password});
 	}
-
     }
 
     if ($args{session_sub}) {
@@ -79,7 +85,7 @@ sub init_from_session {
     my $self = shift;
 
     $self->{account} = $self->{session_sub}->() 
-	|| {uid => 0, username => '', permissions => ['anonymous']};
+	|| {uid => 0, username => '', roles => [], permissions => ['anonymous']};
 
     $self->{acl} = ACL::Lite->new(permissions => $self->{account}->{permissions});
 
@@ -88,7 +94,10 @@ sub init_from_session {
 
 =head2 login
 
-Perform login.
+Perform login. 
+
+Leading and trailing spaces will be removed from
+username and password in advance.
 
 =cut
 
@@ -96,6 +105,13 @@ sub login {
     my ($self, %args) = @_;
     my ($success, $acct);
 
+    # remove leading/trailing spaces from username and password
+    $args{username} =~ s/^\s+//;
+    $args{username} =~ s/\s+$//;
+
+    $args{password} =~ s/^\s+//;
+    $args{password} =~ s/\s+$//;
+
     for my $p (@providers) {
 	if ($acct = $p->login(%args)) {
 	    $self->{session_sub}->('init', $acct);
@@ -144,6 +160,18 @@ sub username {
     return $self->{account}->{username};
 }
 
+=head2 roles
+
+Retrieve roles of this user.
+
+=cut
+
+sub roles {
+    my $self = shift;
+
+    wantarray ? @{$self->{account}->{roles}} : $self->{account}->{roles};
+}
+
 =head2 status
 
 Saves or retrieves status information.
@@ -162,6 +190,65 @@ sub status {
     }
 }
 
+=head2 exists
+
+Check whether account exists.
+
+    if ($acct->exists('shopper@nitesi.biz')) {
+        print "Account exists\n";
+    }
+
+=cut
+
+sub exists {
+    my ($self, $username) = @_;
+
+    return unless defined $username && $username =~ /\S/;
+
+    for my $p (@providers) {
+	if ($p->exists($username)) {
+	    return $p;
+	}
+    }
+}
+
+=head2 password
+
+Changes password for current account:
+
+    $acct->password('nevairbe');
+
+Changes password for other account:
+
+    $acct->password('nevairbe', 'shopper@nitesi.biz');
+
+=cut
+
+sub password {
+    my $self = shift;
+    my ($provider, %args);
+
+    if (@_ == 1) {
+	# new password only
+	unless ($self->{account}->{username}) {
+	    die "Cannot change password for anonymous user";
+	}
+
+	$args{username} = $self->{account}->{username};
+	$args{password} = shift;
+    }
+    else {
+	%args = @_;
+
+	unless ($provider = $self->exists($args{username})) {
+	    die "Cannot change password for user $args{username}.";
+	}
+    }
+
+    $provider->password($self->{password}->password($args{password}),
+			$args{username});
+}
+
 =head2 acl
 
 ACL check, see L<ACL::Lite> for details.
@@ -178,6 +265,20 @@ sub acl {
     }
 }
 
+=head2 value
+
+Retrieve account data.
+
+=cut
+
+sub value {
+    my ($self, $name) = @_;
+
+    if (exists $self->{account}->{$name}) {
+	return $self->{account}->{$name};
+    }
+}
+    
 =head1 AUTHOR
 
 Stefan Hornburg (Racke), <racke@linuxia.de>
